"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Semigroup = exports.Monoid = void 0;
var monoid = _interopRequireWildcard(require("../Monoid.js"));
var semigroup = _interopRequireWildcard(require("../Semigroup.js"));
function _interopRequireWildcard(e, t) { if ("function" == typeof WeakMap) var r = new WeakMap(), n = new WeakMap(); return (_interopRequireWildcard = function (e, t) { if (!t && e && e.__esModule) return e; var o, i, f = { __proto__: null, default: e }; if (null === e || "object" != typeof e && "function" != typeof e) return f; if (o = t ? n : r) { if (o.has(e)) return o.get(e); o.set(e, f); } for (const t in e) "default" !== t && {}.hasOwnProperty.call(e, t) && ((i = (o = Object.defineProperty) && Object.getOwnPropertyDescriptor(e, t)) && (i.get || i.set) ? o(f, t, i) : f[t] = e[t]); return f; })(e, t); }
/**
 * `Semigroup` instance for `Ordering`, returns the left-most non-zero `Ordering`.
 *
 * **Example**
 *
 * ```ts
 * import { Semigroup } from "@effect/typeclass/data/Ordering"
 *
 * console.log(Semigroup.combine(0, -1))
 * // -1
 * console.log(Semigroup.combine(0, 1))
 * // 1
 * console.log(Semigroup.combine(1, -1))
 * // 1
 * ```
 *
 * @category instances
 * @since 0.24.0
 */
const Semigroup = exports.Semigroup = /*#__PURE__*/semigroup.make((self, that) => self !== 0 ? self : that, (self, collection) => {
  let ordering = self;
  if (ordering !== 0) {
    return ordering;
  }
  for (ordering of collection) {
    if (ordering !== 0) {
      return ordering;
    }
  }
  return ordering;
});
/**
 * `Monoid` instance for `Ordering`, returns the left-most non-zero `Ordering`.
 *
 * The `empty` value is `0`.
 *
 * **Example**
 *
 * ```ts
 * import { Monoid } from "@effect/typeclass/data/Ordering"
 *
 * console.log(Monoid.combine(Monoid.empty, -1))
 * // -1
 * console.log(Monoid.combine(Monoid.empty, 1))
 * // 1
 * console.log(Monoid.combine(1, -1))
 * // 1
 * ```
 *
 * @category instances
 * @since 0.24.0
 */
const Monoid = exports.Monoid = /*#__PURE__*/monoid.fromSemigroup(Semigroup, 0);
//# sourceMappingURL=Ordering.js.map